

import com.grapecity.documents.excel.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

public class HighestValues extends CustomFunction
{
	public HighestValues()
	{
		super("HighestValues", FunctionValueType.Text, new Parameter[]
		{
			new Parameter(FunctionValueType.Object),
			new Parameter(FunctionValueType.Number),
			new Parameter(FunctionValueType.Number)
		});

	}

	@Override
	public Object evaluate(Object[] arguments, ICalcContext context)
	{
		if (arguments.length < 3)
		{
			return CalcError.Value;
		}

		Object values = arguments[0];
		if (values instanceof Iterable)
		{
			for (Object x: (Iterable<Object>)(values)) {
				values = x;
				break;
			}
		}
		Object[][] array = values instanceof Object[][] ? (Object[][])values : null;
		if (array == null)
		{
			return CalcError.Value;
		}
		int rowCount = array.length;
		int colCount = (array.length == 0 ? 0 : array[0].length);
		if (rowCount <= 0 || colCount <= 0)
		{
			return CalcError.Value;
		}
		int resultCol = ((Double)(arguments[1])).intValue() - 1;
		if (resultCol < 0 || resultCol >= colCount)
		{
			return CalcError.Num;
		}

		int numberCol = ((Double)(arguments[2])).intValue() - 1;
		if (numberCol < 0 || resultCol >= colCount)
		{
			return CalcError.Num;
		}

		ArrayList<temp> list = new ArrayList<temp>();
		for (int i = 0; i < rowCount; i++)
		{
			String text = array[i][resultCol] == null ? null : array[i][resultCol].toString();
			double number = array[i][numberCol] instanceof Double ? (double)array[i][numberCol] : 0;
			list.add(new temp(text, number));
		}
		Collections.sort(list, (x, y) ->
		{
			if (x.Number > y.Number)
			{
				return -1;
			}
			else if (x.Number == y.Number)
			{
				return 0;
			}
			else
			{
				return 1;
			}
		});
		String result = null;
		int count = Math.min(list.size(), 2);
		for (int i = 0; i < count; i++)
		{
			if (result != null)
			{
				result += ",";
			}else{
				result = "";
			}
			result += list.get(i).Text;
		}

		return result;
	}

	private static class temp
	{
		public String Text;
		public double Number;

		public temp(String text, double number)
		{
			this.Text = text;
			this.Number = number;
		}
	}
}